==> qwac-0.1.4.crate: qwac-0.1.4/src/lib.rs <==
//! This is a simple crate that makes making qwac games easier.
//!
//! It has one feature, `alloc`.  You should enable it if you plan on using liballoc (including if
//! `std` is enabled), otherwise, leave it disabled to save some space.

#![no_std]

pub mod event;
pub mod ffi;

#[cfg(feature = "alloc")]
pub extern crate alloc;

#[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
pub struct Rectangle {
    pub x: u16,
    pub y: u16,
    pub width: u16,
    pub height: u16,
}

#[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
pub struct OpaqueColor {
    pub r: u8,
    pub g: u8,
    pub b: u8,
}

#[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
pub struct Color {
    pub r: u8,
    pub g: u8,
    pub b: u8,
    pub a: u8,
}

pub trait Game {
    /// Create a new Game of this type, starting at the given time in seconds.
    fn new(timestamp: f64) -> Self;

    /// Handle an input event.
    fn event(&mut self, event: event::Event);

    /// Do processing and rendering.  delta tells you the time in seconds between this frame and
    /// the last one.
    fn update(&mut self, frame_milliseconds: u32);
}

#[macro_export]
macro_rules! qwac_game {
    ($game_type:ty) => {
        #[cfg(not(feature = "alloc"))]
        static mut QWAC_RUST_GAME: Option<$game_type> = None;

        #[no_mangle]
        pub extern "C" fn qwac_game_create(timestamp: f64) -> *mut $game_type {
            let game = <$game_type as $crate::Game>::new(timestamp);

            #[cfg(not(feature = "alloc"))]
            {
                unsafe {
                    QWAC_RUST_GAME = None;
                    QWAC_RUST_GAME = Some(game);
                }
                core::ptr::null_mut()
            }

            #[cfg(feature = "alloc")]
            $crate::alloc::boxed::Box::new(game).into_raw()
        }

        #[no_mangle]
        pub extern "C" fn qwac_game_event(
            _game: *mut $game_type,
            event_type: $crate::ffi::EventType,
            player: u8,
            button: u8,
            x: u16,
            y: u16,
        ) {
            #[cfg(not(feature = "alloc"))]
            let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };

            #[cfg(feature = "alloc")]
            let game = unsafe { &mut *_game };

            match event_type {
                $crate::ffi::EventType::GamepadButtonPress => {
                    if let Some(button) = $crate::event::GamepadButton::from_code(button) {
                        $crate::Game::event(
                            game,
                            $crate::event::Event::GamepadButton {
                                button: button,
                                player: player,
                                state: $crate::event::ButtonState::Pressed,
                            },
                        );
                    }
                }
                $crate::ffi::EventType::GamepadButtonRelease => {
                    if let Some(button) = $crate::event::GamepadButton::from_code(button) {
                        $crate::Game::event(
                            game,
                            $crate::event::Event::GamepadButton {
                                button: button,
                                player: player,
                                state: $crate::event::ButtonState::Released,
                            },
                        );
                    }
                }
                _ => (),
            }
        }

        #[no_mangle]
        pub extern "C" fn qwac_game_update(_game: *mut $game_type, frame_milliseconds: u32) {
            #[cfg(not(feature = "alloc"))]
            let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };

            #[cfg(feature = "alloc")]
            let game = unsafe { &mut *_game };

            $crate::Game::update(game, frame_milliseconds);
        }

        #[no_mangle]
        pub extern "C" fn qwac_game_destroy(_game: *mut $game_type) {
            #[cfg(not(feature = "alloc"))]
            unsafe {
                QWAC_RUST_GAME = None;
            }

            #[cfg(feature = "alloc")]
            unsafe {
                $crate::alloc::boxed::Box::from_raw(game);
            }

        }
    };
}

pub fn log(string: &str) {
    let pointer = string.as_ptr();
    let len = string.len() as u32;
    unsafe {
        ffi::qwac_log(pointer, len);
    }
}

pub fn display(width_px: u16, height_px: u16) {
    unsafe {
        ffi::qwac_display(width_px, height_px);
    }
}

pub fn clear_color(color: OpaqueColor) {
    unsafe {
        ffi::qwac_clear_color(color.r, color.g, color.b);
    }
}

pub fn clear() {
    unsafe {
        ffi::qwac_clear();
    }
}

pub fn fill_rectangle(rectangle: Rectangle, color: Color) {
    unsafe {
        ffi::qwac_fill_rectangle(
            rectangle.x,
            rectangle.y,
            rectangle.width,
            rectangle.height,
            color.r,
            color.g,
            color.b,
            color.a,
        );
    }
}

==> qwac-0.2.0.crate: qwac-0.2.0/src/lib.rs <==
//! This is a simple crate that makes making qwac games easier.
//!
//! It has one feature, `alloc`.  You should enable it if you plan on using liballoc (including if
//! `std` is enabled), otherwise, leave it disabled to save some space.

#![no_std]

mod ffi;

pub mod audio;
pub mod event;

#[cfg(feature = "alloc")]
pub extern crate alloc;

#[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
pub struct Rectangle {
    pub x: u16,
    pub y: u16,
    pub width: u16,
    pub height: u16,
}

#[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
pub struct OpaqueColor {
    pub r: u8,
    pub g: u8,
    pub b: u8,
}

#[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
pub struct Color {
    pub r: u8,
    pub g: u8,
    pub b: u8,
    pub a: u8,
}

pub trait Game {
    /// Create a new Game of this type, starting at the given time in seconds.
    fn new(timestamp: f64) -> Self;

    /// Handle an input event.
    fn event(&mut self, event: event::Event);

    /// Do processing and rendering.  delta tells you the time in seconds between this frame and
    /// the last one.
    fn update(&mut self, frame_milliseconds: u32);
}

#[macro_export]
macro_rules! qwac_game {
    ($game_type:ty) => {
        #[cfg(not(feature = "alloc"))]
        static mut QWAC_RUST_GAME: Option<$game_type> = None;

        #[no_mangle]
        pub extern "C" fn qwac_game_create(timestamp: f64) -> *mut $game_type {
            let game = <$game_type as $crate::Game>::new(timestamp);

            #[cfg(not(feature = "alloc"))]
            {
                unsafe {
                    QWAC_RUST_GAME = None;
                    QWAC_RUST_GAME = Some(game);
                }
                core::ptr::null_mut()
            }

            #[cfg(feature = "alloc")]
            $crate::alloc::boxed::Box::new(game).into_raw()
        }

        #[no_mangle]
        pub extern "C" fn qwac_game_event_gamepad(
            _game: *mut $game_type,
            player: u8,
            button: $crate::event::GamepadButton,
            pressed: u8,
        ) {
            #[cfg(not(feature = "alloc"))]
            let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };

            #[cfg(feature = "alloc")]
            let game = unsafe { &mut *_game };

            let state = if pressed == 0 {
                $crate::event::ButtonState::Released
            } else {
                $crate::event::ButtonState::Pressed
            };

            $crate::Game::event(
                game,
                $crate::event::Event::GamepadButton {
                    button,
                    player,
                    state,
                },
            );
        }

        #[no_mangle]
        pub extern "C" fn qwac_game_update(_game: *mut $game_type, frame_milliseconds: u32) {
            #[cfg(not(feature = "alloc"))]
            let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };

            #[cfg(feature = "alloc")]
            let game = unsafe { &mut *_game };

            $crate::Game::update(game, frame_milliseconds);
        }

        #[no_mangle]
        pub extern "C" fn qwac_game_destroy(_game: *mut $game_type) {
            #[cfg(not(feature = "alloc"))]
            unsafe {
                QWAC_RUST_GAME = None;
            }

            #[cfg(feature = "alloc")]
            unsafe {
                $crate::alloc::boxed::Box::from_raw(game);
            }
        }
    };
}

pub fn log(string: &str) {
    let pointer = string.as_ptr();
    let len = string.len() as u32;
    unsafe {
        ffi::qwac_log(pointer, len);
    }
}

pub fn display(width_px: u16, height_px: u16) {
    unsafe {
        ffi::qwac_display(width_px, height_px);
    }
}

pub fn clear_color(color: OpaqueColor) {
    unsafe {
        ffi::qwac_clear_color(color.r, color.g, color.b);
    }
}

pub fn clear() {
    unsafe {
        ffi::qwac_clear();
    }
}

pub fn fill_rectangle(rectangle: Rectangle, color: Color) {
    unsafe {
        ffi::qwac_fill_rectangle(
            rectangle.x,
            rectangle.y,
            rectangle.width,
            rectangle.height,
            color.r,
            color.g,
            color.b,
            color.a,
        );
    }
}
